#include <ThingSpeak.h>
#include <WiFiManager.h>
#include <Adafruit_ADS1X15.h>
#include <Wire.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager WiFi Configuration Magic
#include <esp_task_wdt.h>     //   for the watchdog timer

#define nominal_resistance 10000 //NTC resistance at 25⁰C
#define nominal_temperature 25   // temperature for nominal resistance (almost always 25⁰ C)
#define beta 3435  // The beta coefficient or the B value of the thermistor (usually 3000-4000) check the datasheet for the accurate value.
#define R1 3288   //Value of  resistor used for the voltage divider

#define WDT_TIMEOUT 120000  // 120s watchdog time out
#define CONFIG_FREERTOS_NUMBER_OF_CORES 1  //if 1 core doesn't work, try with 2

esp_task_wdt_config_t twdt_config = {
        .timeout_ms = WDT_TIMEOUT,
        .idle_core_mask = (1 << CONFIG_FREERTOS_NUMBER_OF_CORES) - 1,    // Bitmask of all cores
        .trigger_panic = true,
    };

//Thingspeak channels
//unsigned long myChannelNumber = xxxxxxx; // CO2 Woonkamer en Joris
  unsigned long myChannelNumber = xxxxxxx; // Warmtepomp
//unsigned long myChannelNumber = xxxxxxx; // Bedroom
//unsigned long myChannelNumber = xxxxxxx; // Buiten

//Thingspeak API keys
//const char * myWriteAPIKey = "xxxxxxx"; // CO2 Woonkamer en Joris
  const char * myWriteAPIKey = "xxxxxxx"; // Warmtepomp
//const char * myWriteAPIKey = "xxxxxxx"; // Bedroom
//const char * myWriteAPIKey = "xxxxxxx"; // Buiten

Adafruit_ADS1115 ads;     // this is the 16 bits I2C AD converter

int adcValue;             // needed for temperature measurement
float  NTCresistance = 0; // needed for temperature measurement
float temperature=0;      // needed for temperature measurement

WiFiClient  client;       // needed for thingspeak

void setup(void) {
  Serial.begin(115200);                   // initialise serial
  ThingSpeak.begin(client);               // initialise thinsspeak
  Serial.println("Starting WifiManager");
  WiFiManager wifiManager;
  //wifiManager.resetSettings();
  wifiManager.autoConnect("ESP-NTC");  // Wifi acces point name
  Serial.println("Done setting up Wifi");  
  Serial.println("Configuring WDT...");

  esp_task_wdt_deinit(); //wdt is enabled by default, so we need to deinit it first
  esp_task_wdt_init(&twdt_config); //enable panic so ESP32 restarts
  esp_task_wdt_add(NULL); //add current thread to WDT watch


  //Serial.println("Starting I2C");
  Wire.begin(4,5); // I2C SDA, SCL
  ads.begin(0x48); // ADS1115 I2C address
  //ads.setGain(GAIN_TWOTHIRDS);  // 2/3x gain +/- 6.144V
   ads.setGain(GAIN_ONE);     // 1x gain   +/- 4.096V our range is 0-3.3V
  // ads.setGain(GAIN_TWO);     // 2x gain   +/- 2.048V 
  // ads.setGain(GAIN_FOUR);    // 4x gain   +/- 1.024V
  // ads.setGain(GAIN_EIGHT);   // 8x gain   +/- 0.512V
  // ads.setGain(GAIN_SIXTEEN); // 16x gain  +/- 0.256V
  Serial.println("End of setup");
}

void loop(void) {
  adcValue = ads.readADC_SingleEnded(0);
  NTCresistance = R1/((26400.0/adcValue)-1);  //26400 represents 3.3V/0.000125V
  
  temperature = NTCresistance / nominal_resistance;   // (R/Ro)
  temperature = log(temperature);                     // ln(R/Ro)
  temperature /= beta;                                // 1/B * ln(R/Ro)
  temperature += 1.0 / (nominal_temperature + 273.15);// + (1/To)
  temperature = 1.0 / temperature;                    // Invert
  temperature -= 273.15;                              // convert absolute temp to C
  Serial.print("0,100,");                             // this helps setting the scale for the serial plotter
  Serial.println(temperature);

  if (temperature > -20 || temperature < 150){
    Serial.println("Sending temperature to Thingspeak");
    // Writes just one value to thingSpeak field 1
    int x = ThingSpeak.writeField(myChannelNumber, 1, temperature, myWriteAPIKey);
    if(x == 200){ Serial.println("Channel update successful.");  }
    else{         Serial.println("Problem updating channel. HTTP error code " + String(x));  }
  }
  else{
    Serial.println("Not sending to Thingspeak because of invalid temperature");
  }

  Serial.println("Kicking the watchdog");
  esp_task_wdt_reset();
  delay(60000);// repeat this loop after 60 seconds
}
